# Roadmap

Goal is to build toolkit that would allow rapid application development using EventSourcing as a core architectural pattern.

Being able to build core behaviors and exposing those through GraphQL to the customers is the basic idea.

Goal is multidimensional:
  Autogenerated GraphQL schema
  EventSourcing toolkit
  Idiomatic, human-friendly DSL

## Top to bottom vision
Autogenerated GraphQL used to invoke Commands and Queries, which read from Projections, Aggregates, generate and persist events. Events persisted in EventStore.

## Approach

Minimal prototypes of primitives
Minimal prototype of integrated system
Minimal prototype of application based on system
Refactoring-Improvement-Refactoring

## Challenges

Brain split,
Deliver once,
Transactional consistency,
Nesting transactions,
Persistence,
Serialization,
Validations,
Command composition,
Nested transactions,
Async execution

Event/Command/Schema evolution (behavior, naming, structure),
Identity (Sequent -- Repository for aggregates.
Implements the Unit-Of-Work and Identity-Map patterns to ensure each aggregate is only loaded once per transaction and that you always get the same aggregate instance back.)


## Alternatives

No explicit types ?


## Checklist
  * DX
    - [x] Git Flow
    - [ ] CI
    - [ ] Ruby below 2.5.0 (yield_self usage)
    - [ ] Linter  
    - [ ] Dependency watcher
    - [ ] Badges

  * Command
    - [x] Skeleton without any meaningful structure
    - [x] With inner class input contract
    - [x] With inner class output contract
    - [x] With explicit method input contract
    - [x] With explicit method output contract
    - [x] Without input contract
    - [x] Without output contract
    - [ ] Dry-transaction?
    - [ ] Unified result (Relay?)
    - [ ] Multistep
    - [ ] Composition
    - [ ] Transactions
    - [ ] Docs from comments
    - [ ] Validation with contract
    - [ ] Validation with multiple steps
    - [ ] Validation with custom behavior in the arbitrary point of command
    - [ ] Docs from comments
    - [ ] Code & test generator
  * EventStore
    - [x] In memory store
      - [ ] Serialization and typecasting?
      - [x] Aggregates
      - [ ] Snapshots
      - [ ] Causality
      - [ ] Correlations
      - [ ] Transactions
      - [ ] Projections
      - [ ] Versioning
      - [ ] Locking
      - [ ] Blockchain?
      - [ ] Lifecycle and maintenance tasks
    - [ ] Sequel store
    - [ ] ActiveRecord store
    - [ ] Redis store
    - [ ] Kafka store
    - [ ] IPFS store?
    - [ ] IPDB store?
    - [ ] Docs from comments
  * Event
    - [ ] Hash?
    - [x] Object?
    - [x] Immutability?
    - [x] Aggregate DSL
    - [ ] Correlation DSL
    - [ ] Causation DSL
    - [ ] Module?
    - [ ] With schema?
    - [ ] Naming and code evolution?
    - [ ] Code & test generator
    - [ ] Docs from comments
  * Aggregates
    - [x] Skeleton without any functionality
    - [x] Class instance
    - [x] With events
    - [x] Data transfer DSL
    - [x] To hash
    - [ ] Aggregate from namespaces (*::Events* ?)
    - [ ] Stream DSL
    - [ ] Projection DSL
    - [ ] Conversion to Hash
    - [ ] Hash instance?
    - [ ] With contract?
    - [ ] Code & test generator
    - [ ] Docs from comments
  * Projections
    - [ ] Skeleton without any functionality
  * GraphQL
    - [x] Data type
    - [x] Query
    - [ ] Paginated query
    - [ ] Cursor paginated query
    - [x] Mutation
    - [ ] Unified mutation result (Relay?)
    - [ ] Data generation
    - [ ] Query generation
    - [ ] Mutation generation
    - [ ] Schema from hash?
  * JRuby tests
  * OpalRuby tests

    t = Types::Strict::String.constrained(size: 2..16)
    => #<Dry::Types::Constrained type=#<Dry::Types::Definition primitive=String options={} meta={}> options={:rule=>#<Dry::Logic::Operations::And rules=[#<Dry::Logic::Rule::Predicate predicate=#<Method: Module(Dry::Logic::Predicates::Methods)#type?> options={:args=>[String]}>, #<Dry::Logic::Rule::Predicate predicate=#<Method: Module(Dry::Logic::Predicates::Methods)#size?> options={:args=>[2..16]}>] options={}>} rule=#<Dry::Logic::Operations::And rules=[#<Dry::Logic::Rule::Predicate predicate=#<Method: Module(Dry::Logic::Predicates::Methods)#type?> options={:args=>[String]}>, #<Dry::Logic::Rule::Predicate predicate=#<Method: Module(Dry::Logic::Predicates::Methods)#size?> options={:args=>[2..16]}>] options={}> meta={}>
    > t.to_ast
    => [:constrained, [[:definition, [String, {}]], [:and, [[:predicate, [:type?, [[:type, String], [:input, Undefined]]]], [:predicate, [:size?, [[:size, 2..16], [:input, Undefined]]]]]], {}]]


    irb(main):009:0> require 'dry/types/compiler'
=> false
irb(main):010:0> tc = Dry::Types::Compiler.new(Dry::Types)
=> #<Dry::Types::Compiler:0x007fca038cacd8 @registry=Dry::Types>
irb(main):011:0> Undefined = Dry::Core::Constants::Undefined
=> Undefined
irb(main):012:0> tc.visit([:constrained, [[:definition, [String, {}]], [:and, [[:predicate, [:type?, [[:type, String], [:input, Undefined]]]], [:predicate, [:size?, [[:size, 2..16], [:input, Undefined]]]]]], {}]])
=> #<Dry::Types::Constrained type=#<Dry::Types::Definition primitive=String options={} meta={}> options={:rule=>#<Dry::Logic::Operations::And rules=[#<Dry::Logic::Rule::Predicate predicate=#<Method: Module(Dry::Logic::Predicates::Methods)#type?> options={:args=>[String]}>, #<Dry::Logic::Rule::Predicate predicate=#<Method: Module(Dry::Logic::Predicates::Methods)#size?> options={:args=>[2..16]}>] options={}>} rule=#<Dry::Logic::Operations::And rules=[#<Dry::Logic::Rule::Predicate predicate=#<Method: Module(Dry::Logic::Predicates::Methods)#type?> options={:args=>[String]}>, #<Dry::Logic::Rule::Predicate predicate=#<Method: Module(Dry::Logic::Predicates::Methods)#size?> options={:args=>[2..16]}>] options={}> meta={}>
