require "dry-struct"
require "dry-types"


module Dry
  module Facts
    class Command

      @input_contract  = nil
      @output_contract = nil

      class << self
        def input_contract
          @input_contract if defined?(@input_contract)
        end

        def input_contract= value
          @input_contract = value
        end

        def output_contract
          @output_contract if defined?(@output_contract)
        end

        def output_contract= value
          @output_contract = value
        end

        def call(input)
          input
          .yield_self {|it| _enforce_input_contract_on it }
          .yield_self {|it| _execute_with it.to_hash }
          .yield_self {|it| _enforce_output_contract_on it }
        end

        def define_input_contract &block
          @input_contract =
            Class
            .new(Dry::Struct::Value)
            .yield_self { |k| k.instance_eval(&block) if block }
        end

        def define_output_contract &block
          @output_contract =
            Class
            .new(Dry::Struct::Value)
            .yield_self { |k| k.instance_eval(&block) if block }
        end

        private

        def _enforce_input_contract_on input
          if defined?(self::InputContract)
            self::InputContract
          else
            self.input_contract
          end
          .yield_self {|c| _enforce_contract_if_present(c, input) }
        end

        def _enforce_output_contract_on output
          if defined?(self::OutputContract)
            self::OutputContract
          else
            self.output_contract
          end
          .yield_self {|c| _enforce_contract_if_present(c, output) }
        end

        def _execute_with input
          self.new.call input
        end

        def _enforce_contract_if_present contract, data
          if contract
            contract.new **data
          else
            data
          end
        end

      end

    end
  end
end
