require 'securerandom'

module Dry
  module Facts
    class Event
      include Dry::Equalizer(:id, :data, :metadata)

      @@_aggregate_constructor = false

      attr_reader :data
      attr_reader :metadata

      class << self
        def aggregate_constructor
          @@_aggregate_constructor = true
        end

        def from_h(h)
          Object
          .const_get(h[:metadata][:type][:name])
          .new( data:     h[:data],
                metadata: h[:metadata])
        end
      end
      # object/hash
      # has some infrastructure-related metadata?
      # metadata
        # id
        # event-type-name
        # event-type-version
        # aggregate-id ?
        # correlation-id ?
        # causation-id ?
        # stream-name
      # data

      def initialize  data:     {},
                      metadata: {}
        @metadata = _default_metadata_merged_with(metadata) || Hash.new
        @data     = {}.merge(data) || Hash.new
        freeze
      end

      def id
        @metadata[:id]
      end

      def uuid
        @metadata[:uuid]
      end

      def metadata
        @metadata
      end

      def aggregate_id
        self.metadata[:aggregate_id]
      end

      def to_h
        { data:     self.data,
          metadata: self.metadata }
      end

      private

      def _default_metadata_merged_with(metadata={})
        generated_default_aggregate_id =  if @@_aggregate_constructor
                                            SecureRandom.uuid
                                          end
        generated_default_uuid = SecureRandom.uuid
        generated_default_id   = generated_default_uuid

        { id:           (generated_default_id),
          uuid:         (generated_default_uuid),
          aggregate_id: generated_default_aggregate_id,
          type:         { name:    self.class.name } }
        .merge(metadata || {})
      end

    end
  end
end
