module Dry
  module Facts
    module EventStore
      class InMemoryFactStore

        def initialize
          @serialized_events = []
        end

        def delete_all
          @serialized_events = []
        end

        def aggregate_by_id(klass, id)
          get_events_by_aggregate_id(id)
          .yield_self {|events| klass.build_one_from_events(events)}
        end

        def aggregates_by_ids(klass, ids)
          get_events_by_aggregate_ids(ids)
          .yield_self {|events| klass.build_all_from_events(events)}
        end

        def all_aggregates_of(klass)
          get_events_by_types(klass.event_types)
          .yield_self {|events| klass.build_all_from_events(events)}
        end

        def aggregate_from_event(klass, event)
          get_events_by_aggregate_id(event.to_h[:metadata][:aggregate_id])
          .yield_self {|events| klass.build_one_from_events(events)}
        end

        # commit(event)
        def commit(event)
          # validate & fail
          @serialized_events << event.to_h
          event
        end

        # get_event_by_id(42)
        def get_event_by_id(fact_id)
          # fail if event_id nil?
          # fail if no events found?
          # fail if many events found?
          @serialized_events
          .find_all   {|f|  fact_id == (f && f[:metadata] && f[:metadata][:id])}
          .yield_self {|it| (0 == it.length) ? fail("No event found") : it }
          .yield_self {|it| (1 <  it.length) ? fail("Too may events") : it }
          .first
          .yield_self {|it| deserialize_event(it)}
        end

        def get_events_by_aggregate_id(value)
          @serialized_events
          .find_all   {|e|  value == e[:metadata][:aggregate_id]}
          .map        {|it| deserialize_event(it)}
        end

        def get_events_by_aggregate_ids(value)
          @serialized_events
          .find_all   {|e|  [value].flatten.include? e[:metadata][:aggregate_id]}
          .map        {|it| deserialize_event(it)}
        end


        def get_events_by_types(types)
          @serialized_events
          .find_all   {|f| [types].flatten.include? f[:metadata][:type][:name]}
          .map        {|it| deserialize_event(it)}
        end

        def serialize_event(fact)
          fact
          .to_h
        end

        def deserialize_event(serialized_event)
          Event
          .from_h(serialized_event)
          .freeze
        end

      end
    end
  end
end
